package it.unibas.visualpinco.modello;
import it.unibas.ping.eventi.EventoModificaCollezione;
import it.unibas.ping.eventi.EventoModificaProprieta;
import it.unibas.ping.eventi.Osservabile;
import java.io.File;
import java.util.List;


public class Progetto extends Osservabile{
    
    private File cartellaProgetto;
    private File fileConfig;
    private File cartellaSrc;
    private File cartellaWeb;
    private List azioni = new java.util.ArrayList();
    private String nomeProgetto = "Nessun progetto aperto";
    private String defaultPackage = "";
    
    public Progetto() {
    }

    public File getCartellaProgetto() {
        return cartellaProgetto;
    }

    public void setCartellaProgetto(File cartellaProgetto) {
        this.cartellaProgetto = cartellaProgetto;
    }

    public void setCartellaProgetto(String cartellaProgetto) {
        this.cartellaProgetto = new File(cartellaProgetto);
    }
    
    public File getFileConfig() {
        return fileConfig;
    }

    public void setFileConfig(File fileConfig) {
        this.fileConfig = fileConfig;
        this.setNomeProgetto(fileConfig.getAbsolutePath());
    }

    public File getCartellaSrc() {
        return cartellaSrc;
    }

    public void setCartellaSrc(File cartellaSrc) {
        this.cartellaSrc = cartellaSrc;
    }

    public List getAzioni() {
        return azioni;
    }

    public void setAzioni(List azioni) {
        this.azioni = azioni;
    }
    
    public void addAzione(Azione a){
        if(this.getAzione(a.getNome())!=null){
            throw new IllegalArgumentException("Azione gi esistente");
        }
        this.azioni.add(a);
        this.notificaModificaCollezione(new EventoModificaCollezione(this, "azioni",azioni.size() - 1, EventoModificaCollezione.AGGIUNTA));
    }
    public void removeAzione(String nome){
        for(int i=0;i<azioni.size();i++){
            Azione a = (Azione)azioni.get(i);
            if(a.getNome().equals(nome)){
                this.notificaModificaCollezione(new EventoModificaCollezione(this, "azioni",i, EventoModificaCollezione.ELIMINA));
                this.azioni.remove(i); 
                return;
            }
        }
    }
    public Azione getAzione(String nome){
        for(int i=0;i<azioni.size();i++){
            Azione a = (Azione)azioni.get(i);
            if(a.getNome().equals(nome)){
                return a;
            }
        }
        return null;
    }
    
    public void aggiornaAzione(Azione azione) {
        int indice = this.azioni.indexOf(azione); 
        if (indice == -1) {
            throw new IllegalArgumentException("Azione inesistente: " + azione);
        }
        this.setAzione(indice, azione);
    }

    public void setAzione(int i, Azione azione) {
        if (i < 0 || i >= this.azioni.size()) {
            throw new IndexOutOfBoundsException("Azione inesistente");
        }
        this.azioni.set(i, azione);
        this.notificaModificaCollezione(new EventoModificaCollezione(this, "azioni", i, EventoModificaCollezione.MODIFICA));
    }
    
    public String toString(){
        String s = "";
        s += "---------------\n";
        s += "Progetto Pinco\n";
        s += "---------------\n";
        for(int i=0;i<azioni.size();i++){
            Azione a = (Azione)azioni.get(i);
            s += a.toString();
        }
        s += "---------------";
        return s;
    }
    
    public boolean inizializza(){
        this.clearAzioni();
        this.cartellaSrc = new File(cartellaProgetto.getAbsolutePath() + File.separatorChar + "src");
        if(!cartellaSrc.exists()){
            cartellaSrc.mkdir();
        }
        this.cartellaWeb = new File(cartellaProgetto.getAbsolutePath() + File.separatorChar + "web");
        if(!cartellaWeb.exists()){
            cartellaWeb.mkdir();
        }
        this.setFileConfig(new File(cartellaProgetto.getAbsolutePath() + File.separatorChar + "web" + File.separatorChar + "WEB-INF" + File.separatorChar + "pinco-config.xml"));
        return (fileConfig.exists());
    }

    
    public boolean esisteAzione(String azione){
        azione = azione.replace('.', File.separatorChar);
        azione += ".java";
        File f = new File(cartellaSrc.getAbsolutePath() + File.separatorChar + azione);
        return (f.exists());
    }

    public boolean esisteSchermo(String schermo){
        File f = new File(cartellaWeb.getAbsolutePath() + schermo);
        return (f.exists());
    }
    
    public File getCartellaWeb() {
        return cartellaWeb;
    }

    public void setCartellaWeb(File cartellaWeb) {
        this.cartellaWeb = cartellaWeb;
    }

    public String getNomeProgetto() {
        return nomeProgetto;
    }

    public void setNomeProgetto(String nomeProgetto) {
        this.nomeProgetto = nomeProgetto;
        this.notificaModificaProprieta(new EventoModificaProprieta(this, "nomeProgetto"));
    }
    
    public int getNumeroAzioni(){
        return azioni.size();
    }
    
    private void clearAzioni(){
        this.azioni.clear();
        this.notificaModificaCollezione(new EventoModificaCollezione(this, "azioni", 0, EventoModificaCollezione.ELIMINA));
    }
    public java.util.List toTable(){
        java.util.List listaTabella = new java.util.ArrayList();
        for(int i=0;i<azioni.size();i++){
            
            Azione azione = (Azione)azioni.get(i);
            java.util.List listaEsiti = azione.getEsiti();
                      
            if(listaEsiti.size()==0){
                String[] riga = new String[4];
                riga[0]= azione.getComando().getNome();
                riga[1]= azione.getNome();
                riga[2]="";
                riga[3]="";
                listaTabella.add(riga);
            } else {
                for(int j=0;j<listaEsiti.size();j++){
                    Esito e = (Esito)listaEsiti.get(j);
                    Schermo schermo = e.getSchermo();
                    if(j==0){
                        String[] riga = new String[4];
                        riga[0]= azione.getComando().getNome();
                        riga[1]= azione.getNome();
                        riga[2]= e.getNome();
                        if(schermo==null){
                            riga[3]= "";
                        }else{
                            riga[3]= schermo.getNome();
                        }
                        listaTabella.add(riga);
                    }else{
                        String[] riga = new String[4];
                        riga[0]= "";
                        riga[1]= "";
                        riga[2]= e.getNome();
                        if(schermo==null){
                            riga[3]= "";
                        }else{
                            riga[3]= schermo.getNome();
                        }
                        listaTabella.add(riga);
                    }
                }
            }
            
            
        }
        return listaTabella;
    }
    

    public String getDefaultPackage() {
        //NON ANCORA IMPLEMENTATO
        return defaultPackage;
    }

    public void setDefaultPackage(String defaultPackage) {
        this.defaultPackage = defaultPackage;
    }
    
    public boolean esisteComando(String nome){
        for(int i=0;i<this.azioni.size();i++){
            Azione a = (Azione)azioni.get(i);
            if(a.getComando().getNome().equals(nome)){
                return true;
            }
        }
        return false;
    }
    
    public void caricaDaIndice(Azione azione, Esito esito, int indice){
        java.util.List listaTabella = this.toTable();
        String[] riga = (String[])listaTabella.get(indice);
        String nomeEsito = riga[2];
        String nomeSchermo = riga[3];
        while(riga[0].equals("") && indice >= 0){
            indice--;
            riga = (String[])listaTabella.get(indice);
        }
        String nomeComando = riga[0];
        String nomeAzione = riga[1];
        
        azione = this.getAzione(nomeAzione);
        if(!nomeEsito.equals("")){
            esito = azione.getEsito(nomeEsito);
        }
    }
    
    public Azione azioneDaIndice(int indice){
        java.util.List listaTabella = this.toTable();
        if(indice >= listaTabella.size()){
            return null;
        }
        String[] riga = (String[])listaTabella.get(indice);
        while(riga[0].equals("") && indice >= 0){
            indice--;
            riga = (String[])listaTabella.get(indice);
        }
        String nomeAzione = riga[1];        
        return this.getAzione(nomeAzione);
    }
    public String esitoDaIndice(int indice){
        java.util.List listaTabella = this.toTable();
        if(indice>= listaTabella.size()){
            return "";
        }
        String[] riga = (String[])listaTabella.get(indice);
        return riga[2];
    }
    
    public String controllaProgetto(){
        for(int i=0;i<this.azioni.size();i++){
            Azione a = (Azione)azioni.get(i);
            if(a.getSizeEsiti()==0){
                return "Devi definire almeno un esito per ogni azione!";
            }
        }
        return "";
    }
    
    public Object[] getListaSchermi(){
        java.util.Map mappaSchermi = new java.util.HashMap();
        
        for(int i=0;i<this.azioni.size();i++){
            Azione azione = (Azione)azioni.get(i);
            java.util.List listaEsiti = azione.getEsiti();
            for(int j=0;j<listaEsiti.size();j++){
                Esito esito = (Esito)listaEsiti.get(j);
                Schermo schermo = esito.getSchermo();
                mappaSchermi.put(schermo.getNome(), schermo);
            }
        }
        
        return mappaSchermi.keySet().toArray();
    }
}
