package it.unibas.visualpinco.persistenza;

import it.unibas.ping.framework.Applicazione;
import it.unibas.visualpinco.modello.Azione;
import it.unibas.visualpinco.modello.Comando;
import it.unibas.visualpinco.modello.Esito;
import it.unibas.visualpinco.modello.Progetto;
import it.unibas.visualpinco.modello.Schermo;
import java.util.Iterator;
import java.util.List;
import org.jdom.Element;
import org.jdom.JDOMException;

public class DAOProgetto {
    
    public DAOProgetto() {
    }
    
    public static void caricaProgetto(Progetto progetto) throws DAOException{
        org.jdom.input.SAXBuilder builder = new org.jdom.input.SAXBuilder();
        builder.setValidation(true);
        org.jdom.Document document = null;
        try{
            document = builder.build(progetto.getFileConfig());
            Element root = document.getRootElement();
            Element elAzioni = root.getChild("azioni");
            Applicazione.logFine("Caricato root element");
            List listaAzioni = elAzioni.getChildren("descrittoreAzione");
            Iterator itr = listaAzioni.iterator();
            while(itr.hasNext()) {
                Element elAzione = (Element)itr.next();
                Azione azione = new Azione();
                azione.setNome(elAzione.getAttributeValue("azione"));
                Applicazione.logFine("Caricata azione " + elAzione.getAttributeValue("azione"));
                Comando comando = new Comando();
                comando.setNome(elAzione.getAttributeValue("comando"));
                Applicazione.logFine("Caricato comando " + elAzione.getAttributeValue("comando"));
                azione.setComando(comando);
                azione.setProgetto(progetto);
                
                List listaEsiti = elAzione.getChildren("schermoSuccessivo");
                Iterator iter = listaEsiti.iterator();
                while(iter.hasNext()) {
                    Element elEsito = (Element)iter.next();
                    Esito esito = new Esito();
                    esito.setNome(elEsito.getAttributeValue("esito"));
                    Applicazione.logFine("Caricata esito " + elEsito.getAttributeValue("esito"));
                    Schermo schermo = new Schermo();
                    schermo.setNome(elEsito.getText());
                    Applicazione.logFine("Caricato schermo " + elEsito.getText());
                    esito.setSchermo(schermo);
                    azione.addEsito(esito);
                }
                
                progetto.addAzione(azione);
            }
        }catch(JDOMException jde){
            throw new DAOException("Errore nel parsing del file. " + jde.toString());
        }catch(java.io.IOException ioe){
            throw new DAOException("Errore nel caricamento del file. " + ioe.toString());
        }finally{
            
        }
    }
    
    public static void salvaProgetto(Progetto progetto,String nomeFile) throws DAOException{
        org.jdom.Document document = new org.jdom.Document();
        org.jdom.DocType docType = new org.jdom.DocType("pincoConfig","-//Universita' della Basilicata//DTD pinco//IT","pinco.dtd");
        document.setDocType(docType);
        org.jdom.Element rootElement = new org.jdom.Element("pincoConfig");
        document.setRootElement(rootElement);
        riempiAzioni(progetto,rootElement);
        if(nomeFile==null){
            nomeFile = progetto.getFileConfig().getAbsolutePath();
        }
        salvaDocumento(document, nomeFile);
    }
    
    private static void riempiAzioni(Progetto progetto, Element rootElement){
        java.util.List listaAzioni = progetto.getAzioni();
        org.jdom.Element ePrincipale = new org.jdom.Element("azioni");
        rootElement.addContent(ePrincipale);
        for(int i=0;i<listaAzioni.size();i++){
            Azione azione = (Azione)listaAzioni.get(i);
            
            org.jdom.Element eAzione = new org.jdom.Element("descrittoreAzione");
            eAzione.setAttribute("azione", azione.getNome());
            eAzione.setAttribute("comando", azione.getComando().getNome());
            
            java.util.List listaEsiti = azione.getEsiti();
            for(int j=0;j<listaEsiti.size();j++){
                Esito esito = (Esito)listaEsiti.get(j);
                org.jdom.Element eEsito = new org.jdom.Element("schermoSuccessivo");
                eEsito.setAttribute("esito",esito.getNome());
                eEsito.setText(esito.getSchermo().getNome());
                
                eAzione.addContent(eEsito);
            }
            
            ePrincipale.addContent(eAzione);
        }
    }
    
    private static void salvaDocumento(org.jdom.Document document, String nomeFile) throws DAOException {
        org.jdom.output.XMLOutputter outputter = new org.jdom.output.XMLOutputter();
        org.jdom.output.Format format = org.jdom.output.Format.getPrettyFormat();
        format.setIndent("  ");
        format.setLineSeparator("\n");
        outputter.setFormat(format);
        java.io.FileWriter fileWriter = null;
        try {
            fileWriter = new java.io.FileWriter(nomeFile);
            outputter.output(document, fileWriter);
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            throw new DAOException(ioe);
        } finally {
            if (fileWriter != null) {
                try {
                    fileWriter.close();
                } catch (java.io.IOException ioe) {}
            }
        }
    }
}
